@@ -1,8 +1,8 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2013-2014 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2013-2014 Jason Mehrens. All rights reserved.
+ * Copyright (c) 2013-2015 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013-2015 Jason Mehrens. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -48,10 +48,8 @@ import java.text.MessageFormat;
 import java.util.*;
 import java.util.logging.*;
 import java.util.logging.Formatter;
-import org.junit.AfterClass;
+import org.junit.*;
 import static org.junit.Assert.*;
-import org.junit.BeforeClass;
-import org.junit.Test;
 
 /**
  * The collector formatter tests.
@@ -78,6 +76,10 @@ public class CollectorFormatterTest {
         assertNull(System.getProperty(LOG_CFG_KEY));
         assertEquals(LogManager.class, LogManager.getLogManager().getClass());
     }
+    
+    private static void fullFence() {
+        LogManager.getLogManager().getProperty("");
+    }
 
     @BeforeClass
     public static void setUpClass() throws Exception {
@@ -88,6 +90,16 @@ public class CollectorFormatterTest {
     public static void tearDownClass() throws Exception {
         checkJVMOptions();
     }
+    
+    @Before
+    public void setUp() {
+        fullFence();
+    }
+
+    @After
+    public void tearDown() {
+        fullFence();
+    }
 
     @Test
     public void testFormatHead() {
@@ -387,6 +399,52 @@ public class CollectorFormatterTest {
         assertTrue(f.getTail((Handler) null).length() != 0);
     }
 
+    @Test
+    public void testGetTailExample1() {
+        String p = "{0}{1}{2}{4,choice,-1#|0#|0<... {4,number,integer} more}\n";
+        CollectorFormatter cf = new CollectorFormatter(p);
+        LogRecord r = new LogRecord(Level.WARNING, "warning message");
+        cf.format(r);
+
+        r = new LogRecord(Level.SEVERE, "Encoding failed.");
+        RuntimeException npe = new NullPointerException();
+        StackTraceElement frame = new StackTraceElement("java.lang.String",
+                "getBytes", "String.java", 913);
+        npe.setStackTrace(new StackTraceElement[]{frame});
+        r.setThrown(npe);
+        cf.format(r);
+
+        cf.format(new LogRecord(Level.INFO, "info"));
+        cf.format(new LogRecord(Level.INFO, "info"));
+        cf.getTail((Handler) null);
+    }
+
+    @Test
+    public void testGetTailExample2() {
+        String p = "These {3} messages occurred between\n{7,date,EEE, MMM dd HH:mm:ss:S ZZZ yyyy} and {8,time,EEE, MMM dd HH:mm:ss:S ZZZ yyyy}\n";
+        CollectorFormatter cf = new CollectorFormatter(p);
+        LogRecord min = new LogRecord(Level.SEVERE, "");
+        min.setMillis(1248203502449L);
+        cf.format(min);
+
+        int count = 290;
+        for (int i = 0; i < count; ++i) {
+            LogRecord mid = new LogRecord(Level.SEVERE, "");
+            mid.setMillis(min.getMillis());
+            cf.format(mid);
+        }
+
+        LogRecord max = new LogRecord(Level.SEVERE, "");
+        max.setMillis(1258723764000L);
+        cf.format(max);
+        Object[] args = new Object[9];
+        args[3] = count + 2L;
+        args[7] = min.getMillis();
+        args[8] = max.getMillis();
+        assertEquals(MessageFormat.format(p, args), cf.toString());
+        cf.getTail((Handler) null);
+    }
+
     @Test
     public void testNewDefaultFormatter() {
         String msg = "";
